//*****************************************************************************
//  Description:
//
//	SD24_B, single conversion on a single channel. Interrupt occurs when a
//	single conversion has completed. Test by applying a voltage to channel 2
//	(SD2P0, SD2N0) and setting a breakpoint at the line indicated below.
//
//	Additionally, sampling time can be performed by connecting a logic analyzer
//  or oscilloscope to both MCLK and P1.0. Trigger on the rising edge of P1.0
//  and the time from the rise of P1.0, to the reactivation of MCLK is roughly
//  the sample time.
//
//  ACLK = 32kHz, MCLK = SMCLK = DCO/2 =  4MHz
//
//  //* For minimum Vcc required for SD24_B module - see datasheet          *//
//  //* 100nF cap between Vref and AVss is recommended when using 1.5V REF  *//
//
//                MSP430F673x
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz Crystal
//          --|RST          XOUT|-
//            |                 |
//    Vin+ -->|SD0P0            |
//    Vin- -->|SD0N0        P1.0|--> LED
//            |                 |
//            |            VREF |---+
//            |                 |   |
//            |                 |  -+- 100nF
//            |                 |  -+-
//            |                 |   |
//            |            AVss |---+
//            |                 |
//
//  Built with CCS V5.5
//*****************************************************************************
#include <msp430.h>

/* Unsigned integer to store SD24_B conversion result */
unsigned int results[2];

int main(void) {
    WDTCTL = WDTPW | WDTHOLD;               // Stop WDT

    // Setup LED to show conversion time
    P1DIR |=  BIT0;						   // LED Output
    P1OUT &= ~BIT0;						   // LED Off

    // Setup UCS
    UCSCTL3 |= SELREF_2;                   // Set DCO FLL reference = REFO
    UCSCTL4 |= SELA_2;                     // Set ACLK = REFO

    __bis_SR_register(SCG0);               // Disable the FLL control loop
    UCSCTL0 = 0x0000;                      // Set lowest possible DCOx, MODx
    UCSCTL1 = DCORSEL_5;                   // Select DCO range 16MHz operation
    UCSCTL2 = FLLD_1 | 243;                // Set DCO Multiplier for 8MHz
                                           // (N + 1) * FLLRef = Fdco
                                           // (243 + 1) * 32768 = 8MHz
                                           // Set FLL Div = fDCOCLK/2
    UCSCTL4 = SELA_0  | SELS_4  | SELM_4;  // ACLK = XT1, SCLK = MCLK = DCO/2
    UCSCTL5 = DIVA__1 | DIVS__2 | DIVM__2; // Set /2 dividers for SCLK and MCLK
    __bic_SR_register(SCG0);               // Enable the FLL control loop

    // Worst-case settling time for the DCO when the DCO range bits have been
    // changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
    // UG for optimization.
    // 32 x 32 x 8 MHz / 32,768 Hz = 250000 = MCLK cycles for DCO to settle
    __delay_cycles(250000);

    // Loop until XT1, XT2 & DCO fault flag is cleared
    do {
        UCSCTL7 &= ~(XT2OFFG | XT1LFOFFG | DCOFFG);
        // Clear XT2,XT1,DCO fault flags
        SFRIFG1 &= ~OFIFG;                 // Clear fault flags
    } while (SFRIFG1 & OFIFG);             // Test oscillator fault flag

    SD24BCTL0 = SD24DIV0 | SD24REFS | SD24SSEL_1; // Select internal REF
                         // Select SMCLK/2 (2MHz) as SD24_B clock source

    SD24BCCTL0 |= SD24SNGL;     			// Single conversion, offset binary

    SD24BINCTL0 |= SD24INTDLY_3;			// Interrupt on 1st sample
//    SD24BINCTL0 |= SD24INTDLY_0;			// Interrupt on 4th sample

    SD24BOSR0 = 1023;						// OSR = 1024
//    SD24BOSR0 = 255;						// OSR = 256
//    SD24BOSR0 = 63;							// OSR = 64

    SD24BIE |= SD24IE0;           			// Enable channel 0 interrupt

    __delay_cycles(0x3600);                 // Delay for 1.5V REF startup

    while (1) {
    	SD24BCCTL0 |= SD24SC;               // Set bit to start conversion
        P1OUT |= BIT0;						// LED On, converting
        __bis_SR_register(LPM0_bits | GIE); // Enter LPM0 w/ interrupts
        __no_operation();
    }
}

#pragma vector=SD24B_VECTOR
__interrupt void SD24BISR(void) {
	static unsigned char index = 0;
    switch (SD24BIV) {
        case SD24BIV_SD24OVIFG:             // SD24MEM Overflow
            break;
        case SD24BIV_SD24TRGIFG:            // SD24 Trigger IFG
            break;
        case SD24BIV_SD24IFG0:              // SD24MEM0 IFG
        	P1OUT &= ~BIT0;					// Done converting, LED Off
            results[index++] = SD24BMEML0;  // Save CH0 results (low 16) (clears IFG)
            results[index++] = SD24BMEMH0;  // Save CH0 results (high 16)
            if(index >= 2) {    // Both results are done
            	index = 0;					// SET BREAKPOINT HERE
            }
            break;
        case SD24BIV_SD24IFG1:              // SD24MEM1 IFG
            break;
        case SD24BIV_SD24IFG2:              // SD24MEM2 IFG
            break;
    }
    __bic_SR_register_on_exit(LPM0_bits);   // Exit LPM0
}
